from requests import get
from os import mkdir,path,remove
from tqdm import tqdm
from hashlib import md5
from colorama import init,deinit,Fore

"""
freshlybuiltimagejaanolibrary tokenizer file
status_code for checking the execution status
code   meaning
2000 - token already exist
2001 - token file corrupted
2002 - successful download
2003 - http error
2004 - connection error
2005 - timeout error
2006 - miscellanious error
2007 - building models directory
2008 - signature mismatch 

class token - intiate coroutine check for available tokenizer file
function __init__ : initiate instance of class token
function check_token : checks availability of tokenizer file and intiate other functions according to the need
function get_token : get tokens start downloading in case of absence of tokenizer file 
function token_check : token_check implements md5 hash checksum for checking the corruption of tokenizer file

"""

class token:
    status_code=0
    token_checksum='b8bc863c36b0c5ea638fdb75ea86e1bc'
    curr_dir=path.dirname(path.realpath(__file__))

    def __init__(self):
        self.check_token(self.status_code,self.curr_dir)

    def check_token(self,status_code,curr_dir):
        if path.isfile(curr_dir+"/tokenizer.p"):
            print("tokenizer found")
            self.token_check(self.token_checksum)  
        else:
            print("downloading tokenizer")    
            self.get_token()

    def get_token(self):
        token_url= "https://raw.githubusercontent.com/freshlybuiltongithub/freshylbuiltimagejaano/master/freshlybuiltimagejaano/tokenizer.p"
        response = get(token_url, stream=True)
        try:
            response.raise_for_status()   
        except response.exceptions.HTTPError as errh:
            print ("Http Error:",errh)
            self.status_code=2003
            return self.status_code
        except response.exceptions.ConnectionError as errc:
            print ("Error Connecting:",errc)
            self.status_code=2004
            return self.status_code
        except response.exceptions.Timeout as errt:
            print ("Timeout Error:",errt)
            self.status_code=2005
            return self.status_code
        except response.exceptions.RequestException as err:
            print ("OOps: Something Else",err)
            self.status_code=2006
            return self.status_code
        with open(self.curr_dir+"/"+"tokenizer.p", "wb") as f:
            total_length = int(response.headers.get('content-length'))
            if total_length is None:
                f.write(response.content)
            else:
                chunk_size=512
                for data in tqdm(iterable = response.iter_content(chunk_size),total=total_length/chunk_size, unit = 'KB'):
                    try:
                        f.write(data)
                    except:
                        pass
        self.status_code=2002
        return self.status_code             
    
    def token_check(self,token_checksum):
        model_checksum="tokenizer.p"
        md5_hash=md5()
        model_handler=open(model_checksum,"rb").read()
        md5_hash.update(model_handler)
        hash_code=md5_hash.hexdigest()
        if hash_code==token_checksum:
            init(autoreset=True)
            print("tokenizer signature matched")
            deinit()
            self.status_code=2000
            return self.status_code

        else:
            init(autoreset=True)
            print("tokenizer signature mismatched")
            deinit()
            self.status_code=2001
            print("restarting download")
            remove(self.curr_dir+"/"+"tokenizer.p")
            self.get_token()


#print(token().status_code)



from requests import get
from os import mkdir,path,remove
from tqdm import tqdm
from hashlib import md5
from colorama import init,deinit,Fore

"""
freshlybuiltimagejaano library model downloader
status_code for checking the execution status
code   meaning
1000 - model already exist
1001 - model name incorrect
1002 - download interupted
1003 - successful download
1004 - http error
1005 - connection error
1006 - timeout error
1007 - miscellanious error
1008 - building models directory
1009 - signature mismatch 
"""


"""
class model_downloader 
      checks availability of model directory on run time
      make https request to download model 

      defined functions 
      1 __init__ : declares object of class model_downloader calls model_downloader
      2 download_model: - check directory availability
                        - check model availibility form available model dictionary
                        - returns status code 
      3 start_downloading - make https request to the official github repository for downloading model (continue downloading feature not supported)
      4 hash_signature_match - it generates hashes for the available model to compare with the available models dictionary in download_model function                  

"""
class model_downloader:
    status_code=0000
    
    def __init__(self,model_name,status_code=0):
        self.download_model(model_name,status_code)

    def download_model(self,model_name,status_code):
        dir_path = path.dirname(path.realpath(__file__))
        
        available_models={
            "img_desc":["model_29","57.2MB","3f2502cb4bdb483be654db226e7610be"]
            }
        
        if path.isdir(dir_path+"/models/")==False:
           mkdir(dir_path+"/models")
            
        if model_name in available_models:
            model_name=available_models[model_name]
            if path.isfile(dir_path+"/models/"+model_name[0]+".h5")==False:
                try:
                    print('starting model download')
                    print("don't quit until downloading completes")
                    print('download can take time depending upon your internet conection')
                    init(autoreset=True)
                    print(Fore.YELLOW+" "+model_name[0]+" is of "+model_name[1])
                    choice=input(Fore.YELLOW+"do you wish to download type 'y':")
                    deinit()
                    if (choice=='y'):
                        init(autoreset=True)
                        return self.start_downloading(model_name,dir_path,status_code)
                        deinit()
                    else:
                        init(autoreset=True)
                        print(Fore.RED+'download canceled')
                        deinit()
                        status_code=0000
                        return status_code
                    print('model download successful')
                    self.status_code=1003
                    return status_code
                except:
                    init(autoreset=True)
                    print(Fore.RED+'download interrupted')
                    deinit()
                    try:
                        remove(dir_path+"/models/"+model_name[0] +".h5")
                        self.status_code=1002
                        return status_code
                    except:
                        self.status_code=1002
                        return status_code
            else:
                print('model found')
                print('checking encryption signature')
                self.hash_signature_match(model_name,available_models,dir_path,status_code)
                if self.status_code==1000:
                    return self.status_code
                else:
                    print(self.status_code)
                    print(Fore.CYAN+model_name[0]+" is of "+model_name[1])
                    re_choice=input(Fore.YELLOW+"press 'y' to start re-downloading: ")
                    if re_choice =='y':
                        remove(dir_path+"/models/"+model_name[0] +".h5")
                        self.start_downloading(model_name,dir_path,status_code)
                        self.hash_signature_match(model_name,available_models,dir_path,status_code)
                        return self.status_code
                
        else: 
            print("no reference found for "+model_name)
            self.status_code=1001
            return status_code
    
    
    def start_downloading(self,model_name,dir_path,status_code):
        model_url= "https://raw.githubusercontent.com/freshlybuiltongithub/freshylbuiltimagejaano/master/freshlybuiltimagejaano/models/"
            
        response = get(model_url+model_name[0]+".h5", stream=True)
        try:
            response.raise_for_status()   
        except response.exceptions.HTTPError as errh:
            print ("Http Error:",errh)
            self.status_code=1004
            return self.status_code
        except response.exceptions.ConnectionError as errc:
            print ("Error Connecting:",errc)
            self.status_code=1005
            return self.status_code
        except response.exceptions.Timeout as errt:
            print ("Timeout Error:",errt)
            self.status_code=1006
            return self.status_code
        except response.exceptions.RequestException as err:
            print ("OOps: Something Else",err)
            self.status_code=1007
            return self.status_code
        with open(dir_path+"/models/"+model_name[0] +".h5", "wb") as f:
            total_length = int(response.headers.get('content-length'))
            if total_length is None:
                f.write(response.content)
            else:
                chunk_size=1024
                for data in tqdm(iterable = response.iter_content(chunk_size),total=total_length/chunk_size, unit = 'KB'):
                    try:
                        f.write(data)
                    except:
                        pass
            print("Model download successful")
    
    def hash_signature_match(self,model_name,available_models,dir_path,status_code):
        model_checksum=dir_path+"/models/"+model_name[0]+".h5"
        md5_hash=md5()
        model_handler=open(model_checksum,"rb").read()
        md5_hash.update(model_handler)
        hash_code=md5_hash.hexdigest()
        if hash_code == model_name[2]:
            init(autoreset=True)
            print(Fore.GREEN+"signature matched")
            deinit()
            self.status_code=1000
            return status_code
        else:
            init(autoreset=True)
            print(Fore.RED+"warning signature mismatched, model may not work properly ")
            deinit()
            self.status_code=1009
            return status_code

        
        
"""downloader_debugger"""
#model_name=input("model name: ")
#print(imagebol_model_downloader(model_name).status_code) 



from os import path
from numpy import array,argmax,expand_dims
from os import path
from PIL import Image
from colorama import init,deinit,Fore
from pickle import load
from keras.models import load_model
from keras.applications.xception import Xception
from keras.preprocessing.sequence import pad_sequences
from freshlybuiltimagejaano.downloader import model_downloader
from freshlybuiltimagejaano.get_token import token

'''
image_description class generates description about image

It contains 1.) constructor(), give image path and model name as parameters.
            2.) extract_features(), convert 4 channel image into 3 channels and extract features from image.
            3.) word_for_id(), retrieve Words by index value from tokenizer file.
            4.) generate_desc(), generate descriptions according to features.
            5.) return_desc_text(), this function will return description text to store it into a variable and for further use of this text
'''

class image_description:
	# Constructor of class
    def __init__(self,img_path,model_name):

        available_models={
            "img_desc":"model_29"
            }

        max_length = 32
        dir_path = path.dirname(path.realpath(__file__))


        if path.isfile(dir_path+"/"+"tokenizer.p")==False:
            token()
            tokenizer = load(open(dir_path+"/"+"tokenizer.p","rb"))

        else:
            tokenizer = load(open(dir_path+"/"+"tokenizer.p","rb"))
        print("tokenizer load successful")

        if model_name in available_models:
            model_details=available_models[model_name]
            model_downloader(model_name)
            model = load_model(dir_path+'/models/'+model_details+'.h5')
            xception_model = Xception(include_top=False, pooling="avg")
            init(autoreset=True)
            self.x=self.generate_desc(model, tokenizer,self.extract_features(img_path, xception_model), max_length)
            print(Fore.MAGENTA + self.generate_desc(model, tokenizer,self.extract_features(img_path, xception_model), max_length))
            deinit()
    # Function for returning text  
    def return_desc_text(self):
        return self.x
    # Function for Feature Extraction 
    def extract_features(self,filename, model):
        image = Image.open(filename)
        image = image.resize((299,299))
        image = array(image)
        # converting 4 channel image to 3 channel
        if image.shape[2] == 4: 
            image = image[..., :3]
        image = expand_dims(image, axis=0)
        image = image/127.5
        image = image - 1.0
        feature = model.predict(image)
        return feature
    
    # Function for Retrieving Words by index value from tokenizer file
    def word_for_id(self,integer, tokenizer):
        for word, index in tokenizer.word_index.items():
            if index == integer:
                return word
        return None
    
    # Function for Generate Descriptions 
    def generate_desc(self,model, tokenizer, photo, max_length):
        in_text = 'start'
        for i in range(max_length):
            sequence = tokenizer.texts_to_sequences([in_text])[0]
            sequence = pad_sequences([sequence], maxlen=max_length)
            pred = model.predict([photo,sequence], verbose=0)
            pred = argmax(pred)
            word = self.word_for_id(pred, tokenizer)
            if word is None:
                break
            in_text += ' ' + word
            if word == 'end':
                break
        return in_text

"""debugger"""
#image_name=input('image_name: ')
#image_description(image_name,"img_desc")



from distutils.core import setup
from os import path

DESCRIPTION = """freshlybuiltimagejaano is the library made to allow users to get info about 
                 the input image"""

# The directory containing this file
this_directory = path.abspath(path.dirname(__file__))

# Import the README and use it as the long-description.
try:
    with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

setup(
  name = 'freshlybuiltimagejaano',         
  packages = ['freshlybuiltimagejaano'],   
  version = '0.0.0.7',     
  license='MIT',        
  description = 'Photo k baare m jaano',
  long_description=long_description,
  long_description_content_type="text/markdown",
  author = 'Vishal Sharma',                   
  author_email = 'vishalsharma.gbpecdelhi@gmail.com',      
  url = 'https://github.com/freshlybuiltongithub/freshlybuiltimagejaano',   
  download_url = 'https://github.com/freshlybuiltongithub/freshylbuiltimagejaano/archive/v0.0.0.7.tar.gz',  
  keywords = ['Image', 'Text Generation', 'Text','Image Description from Image'],   
  install_requires=[            
          'Pillow',
          'numpy',
          'tqdm',
          'colorama',
          'keras'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
